{-|
Module      : PUBMED.Client
Description : PubMed API integration
Copyright   : (c) CNRS, 20198
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX
-}


module PUBMED.Client where

import Data.ByteString.Lazy (ByteString)
import Data.Proxy (Proxy(..))
import Servant.API
import Servant.Client
import qualified Data.Text as T
import qualified Network.HTTP.Media as M

import PUBMED.Types (APIKey, Offset, Query, PerPage, WebEnv)

data DB = PUBMED

newtype BsXml = BsXml ByteString
  deriving (Show)

instance Accept BsXml where
  contentType _ = "text" M.// "xml"

instance MimeUnrender BsXml BsXml where
  mimeUnrender _ = Right . BsXml

type PUBMEDAPI =
  -- https://www.ncbi.nlm.nih.gov/books/NBK25499/#chapter4.ESearch
  "esearch.fcgi"
--  :> QueryParam "db" DB
-- not mandatory since the base db is pubmed
  :> QueryParam "api_key" APIKey
  :> QueryParam "term" Query
  :> QueryParam "retstart" Offset
  :> QueryParam "retmax" PerPage
  :> Get '[BsXml] BsXml
  :<|>
  -- https://www.ncbi.nlm.nih.gov/books/NBK25499/#chapter4.ESearch
  "esearch.fcgi"
--  :> QueryParam "db" DB
-- not mandatory since the base db is pubmed
  :> QueryParam "api_key" APIKey
  :> QueryParam "term" Query
  :> QueryParam "retstart" Offset
  :> QueryParam "retmax" PerPage
  :> QueryParam "usehistory" T.Text
  :> QueryParam "WebEnv" WebEnv
  :> Get '[BsXml] BsXml
  :<|>
  "efetch.fcgi"
  :> QueryParam "api_key" APIKey
  :> QueryParam "db" T.Text
  :> QueryParam "rettype" T.Text
  :> QueryParams "id" Integer
  :> Get '[BsXml] BsXml

pubmedApi :: Proxy PUBMEDAPI
pubmedApi = Proxy

search :: Maybe APIKey
       -> Maybe Query
       -> Maybe Offset
       -> Maybe PerPage
       -> ClientM BsXml

searchWithHistory :: Maybe APIKey
                  -> Maybe Query
                  -> Maybe Offset
                  -> Maybe PerPage
                  -> Maybe T.Text
                  -> Maybe WebEnv
                  -> ClientM BsXml

fetch :: Maybe APIKey
      -> Maybe T.Text
      -> Maybe T.Text
      -> [Integer]
      -> ClientM BsXml

search :<|> searchWithHistory :<|> fetch = client pubmedApi
