{-|
Module      : PUBMED.Types
Description : PubMed API integration
Copyright   : (c) CNRS, 20198
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX
-}


module PUBMED.Types where

import Control.Monad.Reader (ReaderT, liftIO)
import Data.Text (Text)
import Control.Monad (when)
import Control.Monad.Reader.Class (asks)


type APIKey = Text
type Offset = Integer
type Query = Text
type PerPage = Integer
type WebEnv = Text

data Config = Config {
    apiKey  :: Maybe APIKey
  , query   :: Query
  , perPage :: Maybe PerPage
  , mWebEnv :: Maybe WebEnv
  -- | If 'True', emit useful debug logs
  , enableDebugLogs :: Bool
  }

type Env = ReaderT Config IO

-- | Prints the debug message only if the verbose logs are enabled.
debugLog :: String -> Env ()
debugLog msg = do
  logs_enabled <- asks enableDebugLogs
  when logs_enabled $ liftIO (putStrLn msg)

-- | This is the default `retmax` value in ESearch.
defaultPerPage :: PerPage
defaultPerPage = 20
