
module Main where

import           Data.Maybe
import           Data.Time
import           PUBMED.Parser as PUBMED
import           Paths_crawlerPubMed

import           Test.Tasty
import           Test.Tasty.HUnit

main :: IO ()
main = defaultMain $ testGroup "PUDMED" [
  testGroup "Parser" parserTests
  ]

parserTests :: [TestTree]
parserTests = [
  testCase "can parse pubmed-29768149.xml" testParse_29768149_xml
  ]

expectedDate :: UTCTime
expectedDate = fromJust $ parseTimeM True defaultTimeLocale "%Y-%m-%d %H:%M:%S UTC" "2018-05-17 00:00:00 UTC"

testParse_29768149_xml :: Assertion
testParse_29768149_xml = do
  fp  <- getDataFileName "test/test-corpus/pubmed-29768149.xml"
  res <- PUBMED.parsePubMedFile fp
  length res @?= 1
  res @?= [PubMed { pubmed_id = 29768149
          , pubmed_article = PubMedArticle {
              pubmed_title = Just "Inhaled Combined Budesonide-Formoterol as Needed in Mild Asthma."
            , pubmed_journal = Just "The New England journal of medicine"
            , pubmed_abstract = ["In patients with mild asthma, as-needed use of an inhaled glucocorticoid plus a fast-acting \946\n\t\t\t\t\t\t2-agonist may be an alternative to conventional treatment strategies.\n\t\t\t\t\t","We conducted a 52-week, double-blind trial involving patients 12 years of age or older with mild asthma. Patients were randomly assigned to one of three regimens: twice-daily placebo plus terbutaline (0.5 mg) used as needed (terbutaline group), twice-daily placebo plus budesonide-formoterol (200 \956g of budesonide and 6 \956g of formoterol) used as needed (budesonide-formoterol group), or twice-daily budesonide (200 \956g) plus terbutaline used as needed (budesonide maintenance group). The primary objective was to investigate the superiority of as-needed budesonide-formoterol to as-needed terbutaline with regard to electronically recorded weeks with well-controlled asthma.","A total of 3849 patients underwent randomization, and 3836 (1277 in the terbutaline group, 1277 in the budesonide-formoterol group, and 1282 in the budesonide maintenance group) were included in the full analysis and safety data sets. With respect to the mean percentage of weeks with well-controlled asthma per patient, budesonide-formoterol was superior to terbutaline (34.4% vs. 31.1% of weeks; odds ratio, 1.14; 95% confidence interval [CI], 1.00 to 1.30; P=0.046) but inferior to budesonide maintenance therapy (34.4% and 44.4%, respectively; odds ratio, 0.64; 95% CI, 0.57 to 0.73). The annual rate of severe exacerbations was 0.20 with terbutaline, 0.07 with budesonide-formoterol, and 0.09 with budesonide maintenance therapy; the rate ratio was 0.36 (95% CI, 0.27 to 0.49) for budesonide-formoterol versus terbutaline and 0.83 (95% CI, 0.59 to 1.16) for budesonide-formoterol versus budesonide maintenance therapy. The rate of adherence in the budesonide maintenance group was 78.9%. The median metered daily dose of inhaled glucocorticoid in the budesonide-formoterol group (57 \956g) was 17% of the dose in the budesonide maintenance group (340 \956g).","In patients with mild asthma, as-needed budesonide-formoterol provided superior asthma-symptom control to as-needed terbutaline, assessed according to electronically recorded weeks with well-controlled asthma, but was inferior to budesonide maintenance therapy. Exacerbation rates with the two budesonide-containing regimens were similar and were lower than the rate with terbutaline. Budesonide-formoterol used as needed resulted in substantially lower glucocorticoid exposure than budesonide maintenance therapy. (Funded by AstraZeneca; SYGMA 1 ClinicalTrials.gov number, NCT02149199 .)."]
            , pubmed_authors = [Author {lastName = Just "O'Byrne", foreName = Just "Paul M", affiliation = Just "From the Firestone Institute for Respiratory Health, St. Joseph's Healthcare and Department of Medicine, Michael G. DeGroote School of Medicine, McMaster University, Hamilton, ON (P.M.O.), and the Institute for Heart and Lung Health, University of British Columbia, Vancouver (J.M.F.) - both in Canada; the Division of Pulmonology, Department of Medicine, University of Cape Town, Cape Town, South Africa (E.D.B.); Airway Disease Section, National Heart and Lung Institute, Imperial College, London (P.J.B.); State Key Laboratory of Respiratory Diseases, First Affiliated Hospital, Guangzhou Medical University, Guangzhou, China (N.Z.); AstraZeneca Research and Development, Gothenburg, Sweden (C.K., C.J., S.I.); AstraZeneca Research and Development, Barcelona (R.L.); and Woolcock Institute of Medical Research, University of Sydney, Sydney (H.K.R.)."},Author {lastName = Just "FitzGerald", foreName = Just "J Mark", affiliation = Just "From the Firestone Institute for Respiratory Health, St. Joseph's Healthcare and Department of Medicine, Michael G. DeGroote School of Medicine, McMaster University, Hamilton, ON (P.M.O.), and the Institute for Heart and Lung Health, University of British Columbia, Vancouver (J.M.F.) - both in Canada; the Division of Pulmonology, Department of Medicine, University of Cape Town, Cape Town, South Africa (E.D.B.); Airway Disease Section, National Heart and Lung Institute, Imperial College, London (P.J.B.); State Key Laboratory of Respiratory Diseases, First Affiliated Hospital, Guangzhou Medical University, Guangzhou, China (N.Z.); AstraZeneca Research and Development, Gothenburg, Sweden (C.K., C.J., S.I.); AstraZeneca Research and Development, Barcelona (R.L.); and Woolcock Institute of Medical Research, University of Sydney, Sydney (H.K.R.)."},Author {lastName = Just "Bateman", foreName = Just "Eric D", affiliation = Just "From the Firestone Institute for Respiratory Health, St. Joseph's Healthcare and Department of Medicine, Michael G. DeGroote School of Medicine, McMaster University, Hamilton, ON (P.M.O.), and the Institute for Heart and Lung Health, University of British Columbia, Vancouver (J.M.F.) - both in Canada; the Division of Pulmonology, Department of Medicine, University of Cape Town, Cape Town, South Africa (E.D.B.); Airway Disease Section, National Heart and Lung Institute, Imperial College, London (P.J.B.); State Key Laboratory of Respiratory Diseases, First Affiliated Hospital, Guangzhou Medical University, Guangzhou, China (N.Z.); AstraZeneca Research and Development, Gothenburg, Sweden (C.K., C.J., S.I.); AstraZeneca Research and Development, Barcelona (R.L.); and Woolcock Institute of Medical Research, University of Sydney, Sydney (H.K.R.)."},Author {lastName = Just "Barnes", foreName = Just "Peter J", affiliation = Just "From the Firestone Institute for Respiratory Health, St. Joseph's Healthcare and Department of Medicine, Michael G. DeGroote School of Medicine, McMaster University, Hamilton, ON (P.M.O.), and the Institute for Heart and Lung Health, University of British Columbia, Vancouver (J.M.F.) - both in Canada; the Division of Pulmonology, Department of Medicine, University of Cape Town, Cape Town, South Africa (E.D.B.); Airway Disease Section, National Heart and Lung Institute, Imperial College, London (P.J.B.); State Key Laboratory of Respiratory Diseases, First Affiliated Hospital, Guangzhou Medical University, Guangzhou, China (N.Z.); AstraZeneca Research and Development, Gothenburg, Sweden (C.K., C.J., S.I.); AstraZeneca Research and Development, Barcelona (R.L.); and Woolcock Institute of Medical Research, University of Sydney, Sydney (H.K.R.)."},Author {lastName = Just "Zhong", foreName = Just "Nanshan", affiliation = Just "From the Firestone Institute for Respiratory Health, St. Joseph's Healthcare and Department of Medicine, Michael G. DeGroote School of Medicine, McMaster University, Hamilton, ON (P.M.O.), and the Institute for Heart and Lung Health, University of British Columbia, Vancouver (J.M.F.) - both in Canada; the Division of Pulmonology, Department of Medicine, University of Cape Town, Cape Town, South Africa (E.D.B.); Airway Disease Section, National Heart and Lung Institute, Imperial College, London (P.J.B.); State Key Laboratory of Respiratory Diseases, First Affiliated Hospital, Guangzhou Medical University, Guangzhou, China (N.Z.); AstraZeneca Research and Development, Gothenburg, Sweden (C.K., C.J., S.I.); AstraZeneca Research and Development, Barcelona (R.L.); and Woolcock Institute of Medical Research, University of Sydney, Sydney (H.K.R.)."},Author {lastName = Just "Keen", foreName = Just "Christina", affiliation = Just "From the Firestone Institute for Respiratory Health, St. Joseph's Healthcare and Department of Medicine, Michael G. DeGroote School of Medicine, McMaster University, Hamilton, ON (P.M.O.), and the Institute for Heart and Lung Health, University of British Columbia, Vancouver (J.M.F.) - both in Canada; the Division of Pulmonology, Department of Medicine, University of Cape Town, Cape Town, South Africa (E.D.B.); Airway Disease Section, National Heart and Lung Institute, Imperial College, London (P.J.B.); State Key Laboratory of Respiratory Diseases, First Affiliated Hospital, Guangzhou Medical University, Guangzhou, China (N.Z.); AstraZeneca Research and Development, Gothenburg, Sweden (C.K., C.J., S.I.); AstraZeneca Research and Development, Barcelona (R.L.); and Woolcock Institute of Medical Research, University of Sydney, Sydney (H.K.R.)."},Author {lastName = Just "Jorup", foreName = Just "Carin", affiliation = Just "From the Firestone Institute for Respiratory Health, St. Joseph's Healthcare and Department of Medicine, Michael G. DeGroote School of Medicine, McMaster University, Hamilton, ON (P.M.O.), and the Institute for Heart and Lung Health, University of British Columbia, Vancouver (J.M.F.) - both in Canada; the Division of Pulmonology, Department of Medicine, University of Cape Town, Cape Town, South Africa (E.D.B.); Airway Disease Section, National Heart and Lung Institute, Imperial College, London (P.J.B.); State Key Laboratory of Respiratory Diseases, First Affiliated Hospital, Guangzhou Medical University, Guangzhou, China (N.Z.); AstraZeneca Research and Development, Gothenburg, Sweden (C.K., C.J., S.I.); AstraZeneca Research and Development, Barcelona (R.L.); and Woolcock Institute of Medical Research, University of Sydney, Sydney (H.K.R.)."},Author {lastName = Just "Lamarca", foreName = Just "Rosa", affiliation = Just "From the Firestone Institute for Respiratory Health, St. Joseph's Healthcare and Department of Medicine, Michael G. DeGroote School of Medicine, McMaster University, Hamilton, ON (P.M.O.), and the Institute for Heart and Lung Health, University of British Columbia, Vancouver (J.M.F.) - both in Canada; the Division of Pulmonology, Department of Medicine, University of Cape Town, Cape Town, South Africa (E.D.B.); Airway Disease Section, National Heart and Lung Institute, Imperial College, London (P.J.B.); State Key Laboratory of Respiratory Diseases, First Affiliated Hospital, Guangzhou Medical University, Guangzhou, China (N.Z.); AstraZeneca Research and Development, Gothenburg, Sweden (C.K., C.J., S.I.); AstraZeneca Research and Development, Barcelona (R.L.); and Woolcock Institute of Medical Research, University of Sydney, Sydney (H.K.R.)."},Author {lastName = Just "Ivanov", foreName = Just "Stefan", affiliation = Just "From the Firestone Institute for Respiratory Health, St. Joseph's Healthcare and Department of Medicine, Michael G. DeGroote School of Medicine, McMaster University, Hamilton, ON (P.M.O.), and the Institute for Heart and Lung Health, University of British Columbia, Vancouver (J.M.F.) - both in Canada; the Division of Pulmonology, Department of Medicine, University of Cape Town, Cape Town, South Africa (E.D.B.); Airway Disease Section, National Heart and Lung Institute, Imperial College, London (P.J.B.); State Key Laboratory of Respiratory Diseases, First Affiliated Hospital, Guangzhou Medical University, Guangzhou, China (N.Z.); AstraZeneca Research and Development, Gothenburg, Sweden (C.K., C.J., S.I.); AstraZeneca Research and Development, Barcelona (R.L.); and Woolcock Institute of Medical Research, University of Sydney, Sydney (H.K.R.)."},Author {lastName = Just "Reddel", foreName = Just "Helen K", affiliation = Just "From the Firestone Institute for Respiratory Health, St. Joseph's Healthcare and Department of Medicine, Michael G. DeGroote School of Medicine, McMaster University, Hamilton, ON (P.M.O.), and the Institute for Heart and Lung Health, University of British Columbia, Vancouver (J.M.F.) - both in Canada; the Division of Pulmonology, Department of Medicine, University of Cape Town, Cape Town, South Africa (E.D.B.); Airway Disease Section, National Heart and Lung Institute, Imperial College, London (P.J.B.); State Key Laboratory of Respiratory Diseases, First Affiliated Hospital, Guangzhou Medical University, Guangzhou, China (N.Z.); AstraZeneca Research and Development, Gothenburg, Sweden (C.K., C.J., S.I.); AstraZeneca Research and Development, Barcelona (R.L.); and Woolcock Institute of Medical Research, University of Sydney, Sydney (H.K.R.)."}]
          }
          , pubmed_date = PubMedDate {pubmedDate_date = expectedDate, pubmedDate_year = 2018, pubmedDate_month = 5, pubmedDate_day = 17}
          }]
